@@ -1,4 +1,8 @@
 from .image import DirectoryImageReader
 from .stack import StackImageReader
-from .timeseries import StackTs, GriddedNcOrthoMultiTs
+from .timeseries import (
+    StackTs,
+    GriddedNcOrthoMultiTs,
+    GriddedNcContiguousRaggedTs,
+)
 from .transpose import write_transposed_dataset
