@@ -5,7 +5,9 @@ Developer's guide
 The ``DirectoryImageReader`` aims to provide an easy to use class to read
 directories of single images, to either create a single image stack file, a
 transposed stack (via ``write_transposed_dataset``), or a cell-based timeseries
-dataset (via the ``repurpose`` method).
+dataset (via the ``repurpose`` method). At the same time, the
+``DirectoryImageReader`` aims to be easy to subclass for specific, more
+complicated datasets.
 
 The advantage over ``xr.open_mfdataset`` is that the reader typically does not
 need to open each dataset to get information on coordinates. Instead, the
@@ -30,8 +32,8 @@ The main routines that should be modified when subclassing are:
 
 * ``_open_dataset``: basic routine for reading data from a file and returning
   it as xr.Dataset
-* ``_latlon_from_dataset``: For manual adaption of how the grid is generated if
-  it cannot be inferred directly from the file.
+* ``_gridinfo_from_dataset``: For manual adaption of how the grid is generated
+  if it cannot be inferred directly from the file.
 * ``_metadata_from_dataset``: If metadata cannot be read from the files.
 * ``_tstamps_in_file``: If the timestamp cannot be inferred from the filename
   but via other info specific to the dataset this can be used to avoid having
@@ -39,12 +41,12 @@ The main routines that should be modified when subclassing are:
 * ``_landmask_from_dataset``: If a landmask is required (only if the ``read``
   function is used), and it cannot be read with ``_open_dataset`` and also not
   with other options. Should not be necessary very often.
-* ``_read_single_file``: normally this calls ``_open_dataset`` and then returns
-  the data as dictionary that maps from variable names to 3d data arrays (numpy
-  or dask). If it is hard to read the data as xr.Dataset, so that overriding
-  `_open_dataset` is not feasible, this could be overriden instead, but then
-  all the other routines for obtaining grid/metadata/landmask info also have to
-  be overriden.
+* ``_read_file``: normally this calls ``_open_dataset`` and then returns the
+  data as dictionary that maps from variable names to 3d data arrays (numpy or
+  dask, dimensions should be time, lat, lon). If it is hard to read the data as
+  xr.Dataset, so that overriding `_open_dataset` is not feasible, this could be
+  overriden instead, but then all the other routines for obtaining
+  grid/metadata/landmask info also have to be overriden.
 
 In the following some examples for subclassing are provided.
 
@@ -130,10 +132,73 @@ quality flags, and the latitude and longitude are only provided as 2D arrays
 with fill values (-9999) at non-land locations (but the grid is a regular
 grid). A reader could look like this::
 
+    from qa4sm_preprocessing.reading.base import GridInfo, _1d_coord_from_2d
+
+
     class SmapSMReader(DirectoryImageReader):
 
         def __init__(self, directory):
-            self.overpass_dict = {
+
+            super().__init__(
+                directory,
+                "SMAP_L3_SM",
+                fmt="%Y%m%d",
+                time_regex_pattern=r"SMAP_L3_SM_P_([0-9]+)_R.*.h5",
+                pattern="**/*.h5",
+                # there are 2 timestamps in each file
+                timestamps=[pd.Timedelta("6H"), pd.Timedelta("18H")]
+            )
+
+        def _read_file(self, fname):
+            # This function only reads the actual data, but not the coordinates
+            # to avoid having to read and construct the coordinates in every
+            # step.
+            sm_arrs = []
+            with h5py.File(fname, "r") as f:
+                for op in ["AM", "PM"]:
+                    sm = self._read_overpass(f, op)
+                    sm_arrs.append(sm)
+            # Now we have read the AM and PM retrievals, but we still need to
+            # concatenate them along the time axis.
+            sm = np.vstack(sm_arrs)
+            return {"SMAP_L3_SM": sm}
+
+        def _read_overpass(self, f, op):
+            # This function reads the data of a single overpass returns it as
+            # numpy array
+            names = self.overpass_dict[op]
+            g = f[names["group"]]
+            sm = np.ma.masked_equal(g[names["sm"]][...], -9999)
+            qc = g[names["qc"]][...]
+            valid = (qc & 1) == 0
+            sm = np.ma.masked_where(~valid, sm).filled(np.nan)
+            return sm[np.newaxis, ...]
+
+        def _gridinfo_from_dataset(self, fname):
+            # This method is called from the constructor of base.ReaderBase and
+            # should return latitude and longitude as xr.DataArrays
+            with h5py.File(fname, "r") as f:
+                g = f["Soil_Moisture_Retrieval_Data_AM"]
+                # reduces the 2D tensor product coordinates to 1D coordinates
+                lat = _1d_coord_from_2d(g["latitude"], 0, fill_value=-9999)
+                lon = _1d_coord_from_2d(g["longitude"], 1, fill_value=-9999)
+            # since we now have 1D coordinates for a 2D array, the grid is
+            # regular
+            gridinfo = GridInfo(lat, lon, "regular")
+            return gridinfo
+
+        def _metadata_from_dataset(self, fname):
+            array_attrs =  {"SMAP_L3_SM": {"long_name": "soil moisture",
+                                           "units": "m^3/m^3"}}
+            global_attrs = {"title": "SMAP level 3 soil moisture"}
+            return global_attrs, array_attrs
+
+        def _dtype_from_dataset(self, fname):
+            return {"SMAP_L3_SM": float}
+
+        @property
+        def overpass_dict(self):
+            return {
                 "AM": {
                     "group": "Soil_Moisture_Retrieval_Data_AM",
                     "sm": "soil_moisture",
@@ -146,49 +211,10 @@ grid). A reader could look like this::
                 }
             }
 
-            super().__init__(
-                directory,
-                "SMAP_L3_SM",
-                fmt="%Y%m%d",
-                time_regex_pattern=r"SMAP_L3_SM_P_([0-9]+)_R.*.h5",
-                pattern="**/*.h5",
-                # there are 2 timestamps in each file
-                timestamps=[pd.Timedelta("6H"), pd.Timedelta("18H")]
-            )
-
-        def _open_dataset(self, fname):
-            sm_arrs = []
-            with h5py.File(fname, "r") as f:
-                for op in ["AM", "PM"]:
-                    sm = self._read_overpass(f, op)
-                    sm_arrs.append(sm)
-            # Now we have read the AM and PM retrievals, but we still need to
-            # concatenate them along a new time axis. We don't have to set the
-            # actual time values though, since they will be inferred from the
-            # filename in combination with the timestamps passed in the
-            # constructor.
-            sm = xr.concat(sm_arrs, dim="time")
-            return sm.to_dataset(name="SMAP_L3_SM")
-
-        def _read_overpass(self, f, op):
-            names = self.overpass_dict[op]
-            g = f[names["group"]]
-            sm = np.ma.masked_equal(g[names["sm"]][...], -9999)
-            qc = g[names["qc"]][...]
-            valid = (qc & 1) == 0
-            sm = np.ma.masked_where(~valid, sm).filled(np.nan)
-            return xr.DataArray(sm, dims=["lat", "lon"])
-
-        def _latlon_from_dataset(self, fname):
-            with h5py.File(fname, "r") as f:
-                g = f["Soil_Moisture_Retrieval_Data_AM"]
-                lat = self.coord_from_2d(g["latitude"], 0, fill_value=-9999)
-                lon = self.coord_from_2d(g["longitude"], 1, fill_value=-9999)
-            return lat, lon
-
-In this example we also adapted ``_latlon_from_dataset``. Instead, we could
-have just read the latitude and longitude in ``_open_dataset`` and added them
-as coordinates to the ``xr.Dataset``, but this way we don't have to read the
+In this example we also adapted ``_gridinfo_from_dataset``,
+``_metadata_from_dataset``, and ``_dtype_from_dataset``. Instead, we could have
+just read the latitude and longitude in ``_open_dataset`` and added them as
+coordinates to the ``xr.Dataset``, but this way we don't have to read the
 latitude and longitude arrays every time we open a file.
 
 More advanced cases
@@ -196,13 +222,12 @@ More advanced cases
 
 If even less information can be obtained from the files, e.g. if the filenames
 don't contain timestamps, it might be necessary to also override
-`_tstamps_in_file`, `_metadata_from_dataset`, or
-`_landmask_from_dataset`. Since these are edge cases, they are not shown in
-detail here, but it works similar to the other examples.
+`_tstamps_in_file`. Since this is an edge cases, it is not shown in detail
+here, but it works similar to the other examples.
 """
 
-import cftime
 import dask
+import dask.array as da
 import datetime
 import numpy as np
 import glob
@@ -210,7 +235,7 @@ from pathlib import Path
 import pandas as pd
 import re
 from tqdm.auto import tqdm
-from typing import Union, Iterable, Sequence, Dict, Tuple, List
+from typing import Union, Iterable, Sequence, Mapping, Tuple, List
 import warnings
 import xarray as xr
 
@@ -228,7 +253,7 @@ class DirectoryImageReader(LevelSelectionMixin, ImageReaderBase):
 
     It can be used to create a single image stack file, a transposed stack (via
     ``write_transposed_dataset``), or a cell-based timeseries dataset (via the
-    repurpose package).
+    ``repurpose`` method).
 
     The advantage over ``xr.open_mfdataset`` is that the reader typically does
     not need to open each dataset to get information on coordinates. Instead,
@@ -325,39 +350,44 @@ class DirectoryImageReader(LevelSelectionMixin, ImageReaderBase):
         data size).
     fill_value : float, optional (default: None)
         Fill values to be masked, e.g. -9999 as a common convention.
-    latname : str, optional (default: "lat")
-        If `locdim` is given (i.e. for non-rectangular grids), this must be the
-        name of the latitude data variable, otherwise must be the name of the
-        latitude coordinate.
-    lonname : str, optional (default: "lon")
-        If `locdim` is given (i.e. for non-rectangular grids), this must be the
-        name of the longitude data variable, otherwise must be the name of the
-        longitude coordinate.
-    timename : str, optional (default: "time")
-        The name of the time coordinate.
-    latdim : str, optional (default: None)
-        The name of the latitude dimension in case it's not the same as the
-        latitude coordinate variable. For curvilinear grids it should be the
-        first dimension of the coordinate dimensions.
-    londim : str, optional (default: None)
-        The name of the longitude dimension in case it's not the same as the
-        longitude coordinate variable. For curvilinear grids it should be the
-        first dimension of the coordinate dimensions.
+    latname : str, optional (default: None)
+        Name of the latitude coordinate array in the dataset. If it is not
+        given, it is inferred from the dataset using CF-conventions.
+    lonname : str, optional (default: None)
+        Name of the longitude coordinate array in the dataset. If it is not
+        given, it is inferred from the dataset using CF-conventions.
+    timename : str, optional (default: None)
+        The name of the time coordinate. Default is "time".
+    ydim : str, optional (default: None)
+        The name of the latitude/y dimension in case it's not the same as the
+        dimension on the latitude array of the dataset. Must be specified if
+        `lat` and `lon` are passed explicitly.
+    xdim : str, optional (default: None)
+        The name of the longitude/x dimension in case it's not the same as the
+        dimension on the longitude array of the dataset. Must be specified if
+        `lat` and `lon` are passed explicitly.
     locdim : str, optional (default: None)
-        The name of the location dimension for non-rectangular grids. If this
-        is given, you *MUST* provide `lonname` and `latname`.
+        The name of the location dimension for non-rectangular grids.
     lat : tuple or np.ndarray, optional (default: None)
         If the latitude can not be inferred from the dataset you can specify it
-        by giving (start, stop, step) or an array of latitude values
+        by giving (start, stop, step) or an array of latitude values. In this
+        case `lon` also has to be specified.
     lon : tuple or np.ndarray, optional (default: None)
         If the longitude can not be inferred from the dataset you can specify
-        it by giving (start, stop, step) or an array of longitude values.
-    curvilinear : bool, optional (default: False)
-        Whether the grid is curvilinear, i.e. is a 2D grid, but not a regular
-        lat-lon grid. In this case, `latname` and `lonname` must be given, and
-        must be names of the variables containing the 2D latitude and longitude
-        values. Additionally, `latdim` and `londim` must be given and will be
-        interpreted as vertical and horizontal dimension.
+        it by giving (start, stop, step) or an array of longitude values. In
+        this case, `lat` also has to be specified.
+    gridtype : str, optional (default: "infer")
+        Type of the grid, one of "regular", "curvilinear", or "unstructured".
+        By default, gridtype is inferred ("infer"). If `locdim` is passed, it
+        is assumed that the grid is unstructured, and that latitude and
+        longitude are 1D arrays. Otherwise, `gridtype` will be set to
+        "curvilinear" if the coordinate arrays are 2-dimensional, and to
+        "regular" if the coordinate arrays are 1-dimensional.
+        Normally gridtype should be set to "infer". Only if the coordinate
+        arrays are 2-dimensional but correspond to a tensor product of two
+        1-dimensional coordinate arrays, it can be set to "regular" explicitly.
+        In this case the 1-dimensional coordinate arrays are inferred from the
+        2-dimensional arrays.
     landmask : xr.DataArray or str, optional (default: None)
         A land mask to be applied to reduce storage size. This can either be a
         xr.DataArray of the same shape as the dataset images with ``False`` at
@@ -405,22 +435,23 @@ class DirectoryImageReader(LevelSelectionMixin, ImageReaderBase):
         skip_missing: bool = False,
         discard_attrs: bool = False,
         fill_value: float = None,
-        latname: str = "lat",
-        lonname: str = "lon",
-        timename: str = "time",
-        latdim: str = None,
-        londim: str = None,
+        latname: str = None,
+        lonname: str = None,
+        timename: str = None,
+        ydim: str = None,
+        xdim: str = None,
         locdim: str = None,
         lat: np.ndarray = None,
         lon: np.ndarray = None,
-        curvilinear: bool = False,
+        gridtype: str = "infer",
+        construct_grid: bool = True,
         landmask: xr.DataArray = None,
         bbox: Iterable = None,
         cellsize: float = None,
-        construct_grid: bool = True,
         average: str = None,
         timestamps: Sequence[pd.Timedelta] = None,
         use_tqdm: bool = True,
+        use_dask: bool = False,
         **open_dataset_kwargs,
     ):
 
@@ -450,15 +481,12 @@ class DirectoryImageReader(LevelSelectionMixin, ImageReaderBase):
             varnames, timeoffsetvarname, rename, level, skip_missing
         )
         self.timeoffsetvarname = timeoffsetvarname
-        if self.timeoffsetvarname is not None:
-            assert timeoffsetunit is not None
-            self.timeoffsetunit = timeoffsetunit.lower()[0]
-            assert self.timeoffsetunit in ["s", "m", "h", "d"]
+        self.timeoffsetunit = timeoffsetunit
         self.rename = rename
         self.level = level
         self.transpose = transpose
-        self.average = average
         self.use_tqdm = use_tqdm
+        self.use_dask = use_dask
         self.fill_value = fill_value
 
         super().__init__(
@@ -467,16 +495,16 @@ class DirectoryImageReader(LevelSelectionMixin, ImageReaderBase):
             timename=timename,
             latname=latname,
             lonname=lonname,
-            latdim=latdim,
-            londim=londim,
+            ydim=ydim,
+            xdim=xdim,
             locdim=locdim,
             lat=lat,
             lon=lon,
+            gridtype=gridtype,
+            construct_grid=construct_grid,
             landmask=landmask,
             bbox=bbox,
             cellsize=cellsize,
-            curvilinear=curvilinear,
-            construct_grid=construct_grid,
         )
 
         ######################################################################
@@ -484,32 +512,18 @@ class DirectoryImageReader(LevelSelectionMixin, ImageReaderBase):
 
         # The next step is to create a map that links filepaths to available
         # timestamps
-        self._file_tstamp_map = {
-            path: self._tstamps_in_file(
-                path, timestamps=timestamps
-            )
+        file_tstamp_map = {
+            path: self._tstamps_in_file(path, timestamps=timestamps)
             for path in filepaths
         }
-        # tstamp_file_map maps each timestamp to the file where it can be found
-        self.tstamp_file_map = {}
-        for fname, tstamps in self._file_tstamp_map.items():
-            for t in tstamps:
-                self.tstamp_file_map[t] = fname
-        self._available_timestamps = sorted(list(self.tstamp_file_map))
 
-        # If we do averaging, the timestamps exposed by the reader is not the
-        # same as the timestamps available on file
-        if self.average is not None:
-            self._output_tstamp_map = {}
-            for tstamp in self._available_timestamps:
-                output_tstamp = self._calculate_averaging_timestamp(tstamp)
-                if output_tstamp in self._output_tstamp_map:
-                    self._output_tstamp_map[output_tstamp].append(tstamp)
-                else:
-                    self._output_tstamp_map[output_tstamp] = [tstamp]
-            self._timestamps = sorted(list(self._output_tstamp_map))
+        if average is None:
+            self.blockreader = _BlockDataReader(self, file_tstamp_map)
         else:
-            self._timestamps = self._available_timestamps
+            self.blockreader = _AveragingBlockDataReader(
+                self, file_tstamp_map, average
+            )
+        self._timestamps = self.blockreader._timestamps
 
         if discard_attrs:
             self.discard_attrs()
@@ -520,9 +534,9 @@ class DirectoryImageReader(LevelSelectionMixin, ImageReaderBase):
     def _open_dataset(self, fname: Union[Path, str]) -> xr.Dataset:
         """Returns data from file as xr.Dataset"""
         # can be overriden for custom datasets
-        return xr.open_dataset(fname, **self.open_dataset_kwargs)
+        return xr.load_dataset(fname, **self.open_dataset_kwargs)
 
-    def _latlon_from_dataset(
+    def _gridinfo_from_dataset(
         self, fname: Union[Path, str]
     ) -> Tuple[np.ndarray, np.ndarray]:
         """Generates lat/lon arrays from a file"""
@@ -530,22 +544,22 @@ class DirectoryImageReader(LevelSelectionMixin, ImageReaderBase):
         if fname == self._example_file:
             ds = self.example_dataset
         else:  # pragma: no cover
-            # Not tested because _open_datset(fname) is tested and this is not
+            # Not tested because _open_dataset(fname) is tested and this is not
             # a typical use case.
-            ds = self._open_dataset(fname)
-        return super()._latlon_from_dataset(ds)
+            ds = self._open_nice_dataset(fname)
+        return super()._gridinfo_from_dataset(ds)
 
     def _metadata_from_dataset(
         self, fname: Union[Path, str]
-    ) -> Tuple[Dict, Dict]:
+    ) -> Tuple[Mapping, Mapping]:
         """Loads the metadata from a file"""
         # can be overriden for custom datasets
         if fname == self._example_file:
             ds = self.example_dataset
         else:  # pragma: no cover
-            # Not tested because _open_datset(fname) is tested and this is not
+            # Not tested because _open_dataset(fname) is tested and this is not
             # a typical use case.
-            ds = self._open_dataset(fname)
+            ds = self._open_nice_dataset(fname)
         return super()._metadata_from_dataset(ds)
 
     def _landmask_from_dataset(
@@ -556,9 +570,23 @@ class DirectoryImageReader(LevelSelectionMixin, ImageReaderBase):
         if fname == self._example_file:
             ds = self.example_dataset
         else:  # pragma: no cover
-            ds = self._open_dataset(fname)
+            ds = self._open_nice_dataset(fname)
         return super()._landmask_from_dataset(ds, landmask)
 
+    def _dtype_from_dataset(self, fname: Union[Path, str]) -> Mapping:
+        if fname == self._example_file:
+            ds = self.example_dataset
+        else:  # pragma: no cover
+            ds = self._open_nice_dataset(fname)
+        return super()._dtype_from_dataset(ds)
+
+    def _shape_from_dataset(self, fname: Union[Path, str]) -> Mapping:
+        if fname == self._example_file:
+            ds = self.example_dataset
+        else:  # pragma: no cover
+            ds = self._open_nice_dataset(fname)
+        return super()._shape_from_dataset(ds)
+
     def _tstamps_in_file(
         self,
         path: Union[Path, str],
@@ -606,105 +634,30 @@ class DirectoryImageReader(LevelSelectionMixin, ImageReaderBase):
             self._example_dataset = self._open_nice_dataset(self._example_file)
         return self._example_dataset
 
-    def _calculate_averaging_timestamp(self, tstamp) -> datetime.datetime:
-        if self.average == "daily":
-            date = tstamp.date()
-            return datetime.datetime(date.year, date.month, date.day)
-        else:
-            raise NotImplementedError("only average='daily' is implemented")
-
     def discard_attrs(self):
         self.global_attrs = {}
         self.array_attrs = {v: {} for v in self.varnames}
 
     def _read_block(
         self, start, end
-    ) -> Dict[str, Union[np.ndarray, dask.array.core.Array]]:
+    ) -> Mapping[str, Union[np.ndarray, da.core.Array]]:
 
-        # reading a block:
-        # - find output timestamps in given range
-        # - if output timestamps == available timestamps
-        #   - for required each file, read the required arrays and stack the
-        #     stacks onto each other
-        # - if averaging has to be done:
-        #   - for each output timestamp:
-        #     - do same procedure as above, then take the mean
-        #   - stack all images
         times = self.tstamps_for_daterange(start, end)
-        if self.average is None:
-            block_dict = self._read_all_files(times, self.use_tqdm)
-        else:
-            # If we have to average multiple images to a single image, we will
-            # read image by image
-            block_dict = {varname: [] for varname in self.varnames}
-            if self.use_tqdm:  # pragma: no branch
-                times = tqdm(times)
-            for tstamp in times:
-                # read all sub-images that have to be averaged later on
-                times_to_read = self._output_tstamp_map[tstamp]
-                tmp_block_dict = self._read_all_files(times_to_read, False)
-                for varname in self.varnames:
-                    block_dict[varname].append(
-                        np.mean(tmp_block_dict[varname], axis=0)
-                    )
-            # now we just have to convert the lists of arrays to array stacks
-            for varname in self.varnames:
-                block_dict[varname] = np.stack(block_dict[varname], axis=0)
-        return block_dict
+        return self.blockreader.read_timestamps(times)
 
-    def _read_single_file(self, fname, tstamps) -> dict:
-        """
-        Reads a single file and returns a dictionary mapping variable names to
-        numpy arrays. Can be overriden in case it's easier to provide this
-        format than xarray datasets.
-        """
-        ds = self._open_nice_dataset(fname)
-        block_dict = {}
-        for varname in self.varnames:
-            arr = ds[varname]
-            if self.timename in arr.dims:
-                # if there are multiple timestamps in each file, reading is
-                # a bit more complicated. In the easiest case, we have as
-                # many timestamps as there are in the file, and we can just
-                # return it.
-                # Otherwise, if time is a coordinate, we can use
-                # .sel(tstamps). If this is also not the case, we need to
-                # find the indices of tstamps in the file
-                if len(tstamps) != len(arr[self.timename]):
-                    if self.timename in arr.coords:
-                        arr = arr.sel({self.timename: tstamps})
-                    else:
-                        all_tstamps = self._file_tstamp_map[fname]
-                        ids = [all_tstamps.index(ts) for ts in tstamps]
-                        arr = arr.isel({self.timename: ids})
-                block_dict[varname] = arr.data
-            else:
-                block_dict[varname] = arr.data[np.newaxis, ...]
-        return block_dict
-
-    def _read_all_files(self, times, use_tqdm):
-        # first we need to find all files that we have to visit, and remember
-        # the timestamps that we need from this file
-        file_tstamp_map = {}
-        for tstamp in times:
-            fname = self.tstamp_file_map[tstamp]
-            if fname in file_tstamp_map:
-                file_tstamp_map[fname].append(tstamp)
-            else:
-                file_tstamp_map[fname] = [tstamp]
-
-        # now we can open each file and extract the timestamps we need
-        block_dict = {varname: [] for varname in self.varnames}
-        iterator = file_tstamp_map.items()
-        if use_tqdm:  # pragma: no branch
-            iterator = tqdm(iterator)
-        for fname, tstamps in iterator:
-            _blockdict = self._read_single_file(fname, tstamps)
-            for varname in block_dict:
-                block_dict[varname].append(_blockdict[varname])
-        for varname in self.varnames:
-            block_dict[varname] = np.vstack(block_dict[varname])
-        return block_dict
+    def _read_file(
+        self, fname: Union[Path, str]
+    ) -> Mapping[str, Union[np.ndarray, da.core.Array]]:
+        # this function reads a xr.Dataset and converts it to the dictionary
+        # format that is used internally
+        dims = self.get_dims()
+        ds = self._open_nice_dataset(fname)[self.varnames]
+        # since we pass the data as dictionary of numpy arrays, we need to make
+        # sure that the dimensions are in the order in which they are expected.
+        actual_dims = [d for d in dims if d in list(ds.dims)]
+        ds = ds.transpose(*actual_dims)
+        data = {v: self._fix_ndim(ds[v].data) for v in self.varnames}
+        return data
 
     def _make_nicer_ds(self, ds: xr.Dataset) -> xr.Dataset:
         if self.fill_value is not None:
@@ -716,29 +669,8 @@ class DirectoryImageReader(LevelSelectionMixin, ImageReaderBase):
             ds = ds.rename(self.rename)
         return ds
 
-    def _convert_timeoffset(self, ds, fname) -> xr.Dataset:
-        if self.timeoffsetvarname is not None:
-            var = self.timeoffsetvarname
-            assert "since" not in self.timeoffsetunit, (
-                "time offset units must be relative to current timestamp"
-            )
-            timestamp = self._tstamps_in_file(fname)[0]
-            start_date = cftime.num2date(0, f"days since {str(timestamp)}")
-            start = cftime.date2num(start_date, "days since 1900-01-01")
-
-            conversion = {
-                "s": 86400, "m": 24*60, "h": 24, "d": 1
-            }[self.timeoffsetunit]
-            # start = pd.to_datetime(timestamp).to_julian_date()
-            time = start + ds[var] / conversion
-            ds[var] = time
-            ds[var].attrs["units"] = "days since 1900-01-01"
-            ds[var].attrs["long_name"] = "Observation time"
-        return ds
-
     def _open_nice_dataset(self, fname) -> xr.Dataset:
         ds = self._make_nicer_ds(self._open_dataset(fname))
-        ds = self._convert_timeoffset(ds, fname)
         return ds
 
     def _fix_varnames_rename_level(
@@ -766,10 +698,7 @@ class DirectoryImageReader(LevelSelectionMixin, ImageReaderBase):
         #
         # (note the missing LAI, since we derive the map from the variables
         # in the file)
-        if isinstance(varnames, str):
-            varnames = [varnames]
-        if timeoffsetvarname is not None and timeoffsetvarname not in varnames:
-            varnames.append(timeoffsetvarname)
+        varnames = self._maybe_add_varnames(varnames, [timeoffsetvarname])
         level = self.normalize_level(level, varnames)
         if skip_missing:
             # Be careful: skip_missing only works if _open_dataset does not do
@@ -841,3 +770,184 @@ class DirectoryImageReader(LevelSelectionMixin, ImageReaderBase):
                     )
             varnames = new_varnames
         return varnames, rename, level
+
+
+class _BlockDataReader:
+    # class for internal use to move some of the complexity of the mapping of
+    # timestamps to files out of the reader, in the hope to make it easier for
+    # people to subclass the DirectoryImageReader.
+
+    def __init__(
+        self, directoryreader, file_tstamp_map, use_tqdm: bool = True
+    ):
+        self.directoryreader = directoryreader
+        self._file_tstamp_map = file_tstamp_map
+        self._use_tqdm = use_tqdm  # can only be used to turn it off
+        # invert file_tstamp_map
+        self.tstamp_file_map = {}
+        for fname, tstamps in self._file_tstamp_map.items():
+            for t in tstamps:
+                self.tstamp_file_map[t] = fname
+
+    @property
+    def _timestamps(self):
+        return sorted(list(self.tstamp_file_map))
+
+    @property
+    def varnames(self):
+        return self.directoryreader.varnames
+
+    @property
+    def use_dask(self):
+        return self.directoryreader.use_dask
+
+    @property
+    def dtype(self):
+        return self.directoryreader.dtype
+
+    @property
+    def use_tqdm(self):
+        # using tqdm for a progressbar only makes sense if we do not use dask
+        # otherwise it's better to use dask's diagnostic tools
+        return (
+            self.directoryreader.use_tqdm
+            and self._use_tqdm
+            and not self.use_dask
+        )
+
+    def get_file_tstamp_map(self, timestamps):
+        # Here we need to get the file_tstamp_map limited to a selection of
+        # timestamps. To get it, we use the inverse map (tstamp_file_map),
+        # select our timestamps, and invert it again.
+        file_tstamp_map = {}
+        for tstamp in timestamps:
+            fname = self.tstamp_file_map[tstamp]
+            if fname in file_tstamp_map:
+                file_tstamp_map[fname].append(tstamp)
+            else:
+                file_tstamp_map[fname] = [tstamp]
+        return file_tstamp_map
+
+    def read_timestamps(self, timestamps: Sequence[datetime.datetime]):
+        block_dict = {v: [] for v in self.varnames}
+        iterator = self.get_file_tstamp_map(timestamps).items()
+        if self.use_tqdm:
+            iterator = tqdm(iterator)
+        for fname, tstamps in iterator:
+            cur_blockdict = self.read_timestamps_from_file(fname, tstamps)
+            for v in self.varnames:
+                block_dict[v].append(cur_blockdict[v])
+        return self._assemble_blockdict(block_dict)
+
+    def read_timestamps_from_file(self, fname, timestamps):
+        # this just handles dask, the actual implementation is in
+        # self._read_single_file_timestamps
+        if self.use_dask:
+            ntime = len(timestamps)
+            shape = self.directoryreader.get_blockshape(ntime)
+            delayed_blockdict = dask.delayed(
+                self._read_single_file_timestamps
+            )(fname, timestamps)
+            blockdict = {
+                v: dask.array.from_delayed(
+                    delayed_blockdict[v], shape=shape, dtype=self.dtype[v]
+                )
+                for v in self.varnames
+            }
+            return blockdict
+        else:
+            return self._read_single_file_timestamps(fname, timestamps)
+
+    def _read_single_file_timestamps(self, fname, timestamps):
+        data = self.directoryreader._read_file(fname)
+        ntime_should = len(timestamps)
+        actual_ntime = data[self.varnames[0]].shape[0]
+        if ntime_should < actual_ntime:
+            # There are some timestamps in the data that we are not interested
+            # in. We assume that the timestamps in the file are ordered, so we
+            # can just check the indices of the timestamps we want and extract
+            # the corresponding data.
+            tstamps_in_file = self._file_tstamp_map[fname]
+            ids = [tstamps_in_file.index(ts) for ts in timestamps]
+            data = {v: data[v][ids, ...] for v in data}
+        return data
+
+    def _assemble_blockdict(
+        self,
+        blockdict: Mapping[str, Sequence[Union[np.ndarray, da.core.Array]]],
+    ) -> Mapping[str, Union[np.ndarray, da.core.Array]]:
+        if self.use_dask:
+            vstack = da.vstack
+        else:
+            vstack = np.vstack
+        return {v: vstack(blockdict[v]) for v in blockdict}
+
+
+class _AveragingBlockDataReader(_BlockDataReader):
+    # The averaging reader does similar things than the block reader, but here
+    # we modify the _file_tstamp_map, to map from output (averaged) timestamps
+    # to timestamps on file. This means we will have the mapping the other way
+    # around, so we adapt ``get_file_tstamp_map`` and ``_timestamps`` to return
+    # the things we want.
+    # In read_timestamps_from_file, we just call another block reader to do the
+    # dirty work, and then average what we got from it.
+
+    def __init__(self, directoryreader, file_tstamp_map, average):
+        output_tstamp_map = {}
+        available_timestamps = sum(
+            [tstamps for tstamps in file_tstamp_map.values()], start=[]
+        )
+        for tstamp in available_timestamps:
+            output_tstamp = self.calculate_averaging_timestamp(average, tstamp)
+            if output_tstamp in output_tstamp_map:
+                output_tstamp_map[output_tstamp].append(tstamp)
+            else:
+                output_tstamp_map[output_tstamp] = [tstamp]
+        self._averaging_timestamps = sorted(list(output_tstamp_map))
+        super().__init__(directoryreader, output_tstamp_map)
+
+        # To remember the underlying mapping of actual timestamps to files, we
+        # will additionally initialize a "normal" block reader, which can then
+        # read the actual data.
+        self.blockreader = _BlockDataReader(
+            directoryreader, file_tstamp_map, use_tqdm=False
+        )
+
+    @property
+    def _timestamps(self):
+        return self._averaging_timestamps
+
+    def get_file_tstamp_map(self, timestamps):
+        # in this case we want to return all entries for the given timestamps,
+        # which is easier than for the _BlockDataReader
+        return {t: self._file_tstamp_map[t] for t in timestamps}
+
+    def calculate_averaging_timestamp(
+        self, average, tstamp
+    ) -> datetime.datetime:
+        if average == "daily":
+            date = tstamp.date()
+            return datetime.datetime(date.year, date.month, date.day)
+        else:
+            raise NotImplementedError("only average='daily' is implemented")
+
+    def read_timestamps_from_file(self, fname, timestamps):
+
+        # The trick here is the following: AveragedTimestampReader is a
+        # BlockDataReader, therefore it will handle the looping over the
+        # individual output timestamps using the read_method defined in the
+        # BlockDataReader (which also handles all the dask stuff).
+        # But it also has it's own BlockReader instance, which is now called to
+        # do the dirty work.
+        data = self.blockreader.read_timestamps(timestamps)
+
+        # now we only have to do the averaging and return the data
+        for v in self.varnames:
+            # otherwise lots of warnings for mean of empty slice
+            warnings.filterwarnings(
+                action="ignore", message="Mean of empty slice"
+            )
+            # take mean along first dimension (time dimension), but add it
+            # again so we have it in the shape the data is expected later on.
+            data[v] = np.nanmean(data[v], axis=0)[np.newaxis, ...]
+        return data
