@@ -37,97 +37,72 @@ import pytest
 import shutil
 import xarray as xr
 
-from pygeogrids.grids import BasicGrid
 
 from qa4sm_preprocessing.reading import (
     DirectoryImageReader,
 )
 from qa4sm_preprocessing.reading.write import write_images
 
-# this is defined in conftest.py
-from pytest import test_data_path
+from .utils import validate_reader
 
-
-def validate_reader(reader, dataset, timename="time", grid=True):
-    expected_timestamps = dataset.indexes[timename]
-    assert len(reader.timestamps) == len(expected_timestamps)
-    assert np.all(list(reader.timestamps) == expected_timestamps)
-
-    if grid:
-        assert isinstance(reader.grid, BasicGrid)
-    else:
-        assert reader.grid is None
-
-    # check if read_block gives the correct result for the first image
-    expected_img = dataset.sel(**{timename: reader.timestamps[0]})
-    img = reader.read_block(reader.timestamps[0], reader.timestamps[0])
-    assert expected_img.attrs == reader.global_attrs
-    assert expected_img.attrs == img.attrs
-    assert sorted(list(expected_img.data_vars)) == sorted(list(img.data_vars))
-    for var in img.data_vars:
-        np.testing.assert_equal(
-            img[var].values.squeeze(), expected_img[var].values.squeeze()
-        )
-
-    # check if read_block gives the correct result for the full block
-    block = reader.read_block()
-    assert dataset.attrs == reader.global_attrs
-    assert dataset.attrs == block.attrs
-    assert sorted(list(dataset.data_vars)) == sorted(list(block.data_vars))
-    for var in block.data_vars:
-        np.testing.assert_equal(
-            block[var].values.squeeze(), dataset[var].values.squeeze()
-        )
-
-
-def test_directory_image_reader_basic(synthetic_test_args):
+def test_directory_image_reader_basic(synthetic_test_args, test_output_path):
     ds, kwargs = synthetic_test_args
-    write_images(ds, test_data_path / "synthetic", "synthetic")
+    write_images(ds, test_output_path / "synthetic", "synthetic")
     reader = DirectoryImageReader(
-        test_data_path / "synthetic",
+        test_output_path / "synthetic",
         ["X", "Y"],
         fmt="synthetic_%Y%m%dT%H%M.nc",
         **kwargs
     )
     validate_reader(reader, ds)
-    shutil.rmtree(test_data_path / "synthetic")
 
 
-def test_directory_image_reader_incomplete_varnames(synthetic_test_args):
+def test_directory_image_reader_use_dask(synthetic_test_args, test_output_path):
+    ds, kwargs = synthetic_test_args
+    write_images(ds, test_output_path / "synthetic", "synthetic")
+    reader = DirectoryImageReader(
+        test_output_path / "synthetic",
+        ["X", "Y"],
+        fmt="synthetic_%Y%m%dT%H%M.nc",
+        use_dask=True,
+        **kwargs
+    )
+    validate_reader(reader, ds)
+
+
+def test_directory_image_reader_incomplete_varnames(synthetic_test_args, test_output_path):
     # tests whether reading a single variable also works as expected
     ds, kwargs = synthetic_test_args
     ds = ds[["X"]]
-    write_images(ds, test_data_path / "synthetic", "synthetic")
+    write_images(ds, test_output_path / "synthetic", "synthetic")
     reader = DirectoryImageReader(
-        test_data_path / "synthetic", ["X"], fmt="synthetic_%Y%m%dT%H%M.nc", **kwargs
+        test_output_path / "synthetic", ["X"], fmt="synthetic_%Y%m%dT%H%M.nc", **kwargs
     )
     validate_reader(reader, ds)
-    shutil.rmtree(test_data_path / "synthetic")
 
 
-def test_directory_image_reader_renaming(synthetic_test_args):
+def test_directory_image_reader_renaming(synthetic_test_args, test_output_path):
     # tests whether renaming works
     ds, kwargs = synthetic_test_args
     test_ds = ds.rename({"X": "newX", "Y": "newY"})
-    write_images(test_ds, test_data_path / "synthetic", "synthetic")
+    write_images(test_ds, test_output_path / "synthetic", "synthetic")
     reader = DirectoryImageReader(
-        test_data_path / "synthetic",
+        test_output_path / "synthetic",
         ["X", "Y"],
         rename={"newX": "X", "newY": "Y"},
         fmt="synthetic_%Y%m%dT%H%M.nc",
         **kwargs
     )
     validate_reader(reader, ds)
-    shutil.rmtree(test_data_path / "synthetic")
 
 
-def test_directory_image_reader_renaming_level(synthetic_test_args):
+def test_directory_image_reader_renaming_level(synthetic_test_args, test_output_path):
     ds, kwargs = synthetic_test_args
     new_ds = xr.concat((ds, ds * 2), dim="level").transpose(..., "level")
     new_ds = new_ds.rename({"X": "newX", "Y": "newY"})
-    write_images(new_ds, test_data_path / "synthetic", "synthetic")
+    write_images(new_ds, test_output_path / "synthetic", "synthetic")
     reader = DirectoryImageReader(
-        test_data_path / "synthetic",
+        test_output_path / "synthetic",
         ["X", "Y"],
         fmt="synthetic_%Y%m%dT%H%M.nc",
         rename={"newX": "X", "newY_0": "Y"},
@@ -136,93 +111,88 @@ def test_directory_image_reader_renaming_level(synthetic_test_args):
         **kwargs
     )
     validate_reader(reader, ds)
-    shutil.rmtree(test_data_path / "synthetic")
 
 
-def test_directory_image_reader_missing_varname(synthetic_test_args):
+def test_directory_image_reader_missing_varname(synthetic_test_args, test_output_path):
     # tests whether skipping missing variables works
     ds, kwargs = synthetic_test_args
     ds = ds[["X"]]
-    write_images(ds, test_data_path / "synthetic", "synthetic")
+    write_images(ds, test_output_path / "synthetic", "synthetic")
     with pytest.warns(
         UserWarning, match="Skipping variable 'Y' because it does not exist!"
     ):
         # skipping a variable raises a warning
         reader = DirectoryImageReader(
-            test_data_path / "synthetic",
+            test_output_path / "synthetic",
             ["X", "Y"],
             fmt="synthetic_%Y%m%dT%H%M.nc",
             skip_missing=True,
             **kwargs
         )
     validate_reader(reader, ds)
-    shutil.rmtree(test_data_path / "synthetic")
 
 
-def test_directory_image_reader_missing_varname_level_rename(synthetic_test_args):
+def test_directory_image_reader_missing_varname_level_rename(synthetic_test_args, test_output_path):
     # tests whether skipping missing variables works
     ds, kwargs = synthetic_test_args
-    ds = ds[["X"]]
-    new_ds = xr.concat((ds, ds * 2), dim="level").transpose(..., "level")
+    new_ds = xr.concat((ds[["X"]], ds[["X"]] * 2), dim="level").transpose(..., "level")
+    new_ds["Y"] = ds["Y"]
     new_ds = new_ds.rename({"X": "newX"})
-    write_images(new_ds, test_data_path / "synthetic", "synthetic")
+    write_images(new_ds, test_output_path / "synthetic", "synthetic")
     with pytest.warns(
-        UserWarning, match="Skipping variable 'Y' because it does not exist!"
+        UserWarning, match="Skipping variable 'Z' because it does not exist!"
     ):
         # skipping a variable raises a warning
         reader = DirectoryImageReader(
-            test_data_path / "synthetic",
-            ["X", "Y"],
+            test_output_path / "synthetic",
+            ["X", "Y", "Z"],
             fmt="synthetic_%Y%m%dT%H%M.nc",
             skip_missing=True,
-            rename={"newX_0": "X", "newY": "Y"},
-            level={"newX": {"level": [0]}, "newY": {"level": 0}},
+            rename={"newX_0": "X", "newZ": "Z"},
+            level={"newX": {"level": [0]}, "newZ": {"level": 0}},
             **kwargs
         )
     validate_reader(reader, ds)
-    shutil.rmtree(test_data_path / "synthetic")
 
 
-def test_directory_image_reader_pattern(synthetic_test_args):
+def test_directory_image_reader_pattern(synthetic_test_args, test_output_path):
     # tests whether skipping files that don't match the pattern works
     ds, kwargs = synthetic_test_args
-    write_images(ds, test_data_path / "synthetic", "synthetic")
+    write_images(ds, test_output_path / "synthetic", "synthetic")
     # add another netCDF file with a different filename that would make the
     # reader fail
-    ds.to_netcdf(test_data_path / "synthetic" / "full_stack.nc")
+    ds.to_netcdf(test_output_path / "synthetic" / "full_stack.nc")
     reader = DirectoryImageReader(
-        test_data_path / "synthetic",
+        test_output_path / "synthetic",
         ["X", "Y"],
         fmt="synthetic_%Y%m%dT%H%M.nc",
         pattern="synthetic*.nc",
         **kwargs
     )
     validate_reader(reader, ds)
-    shutil.rmtree(test_data_path / "synthetic")
 
 
-def test_directory_image_reader_time_regex_pattern(synthetic_test_args):
+def test_directory_image_reader_time_regex_pattern(synthetic_test_args, test_output_path):
     # tests whether the time regex pattern argument works
     ds, kwargs = synthetic_test_args
-    write_images(ds, test_data_path / "synthetic", "synthetic")
+    write_images(ds, test_output_path / "synthetic", "synthetic")
     reader = DirectoryImageReader(
-        test_data_path / "synthetic",
+        test_output_path / "synthetic",
         ["X", "Y"],
         fmt="%Y%m%dT%H%M",
         time_regex_pattern="synthetic_([0-9T]+).nc",
         **kwargs
     )
     validate_reader(reader, ds)
-    shutil.rmtree(test_data_path / "synthetic")
 
 
-def test_directory_image_reader_select_level(synthetic_test_args):
+def test_directory_image_reader_select_level(synthetic_test_args, test_output_path):
     # tests whether selecting from a level dimension works
     ds, kwargs = synthetic_test_args
     ds = xr.concat((ds, ds * 2), dim="level").transpose(..., "level")
-    write_images(ds, test_data_path / "synthetic", "synthetic")
+    write_images(ds, test_output_path / "synthetic", "synthetic")
     reader = DirectoryImageReader(
-        test_data_path / "synthetic",
+        test_output_path / "synthetic",
         ["X", "Y"],
         fmt="synthetic_%Y%m%dT%H%M.nc",
         level={"X": {"level": 0}, "Y": {"level": 1}},
@@ -233,32 +203,30 @@ def test_directory_image_reader_select_level(synthetic_test_args):
     newds = xr.merge((X, Y))
     newds.attrs = ds.attrs
     validate_reader(reader, newds)
-    shutil.rmtree(test_data_path / "synthetic")
 
 
-def test_directory_image_reader_select_level_single_var(synthetic_test_args):
+def test_directory_image_reader_select_level_single_var(synthetic_test_args, test_output_path):
     # tests whether selecting from a level dimension for a single variable with
     # shorthand notation works
     ds, kwargs = synthetic_test_args
     ds = xr.concat((ds, ds * 2), dim="level").transpose(..., "level")
-    write_images(ds, test_data_path / "synthetic", "synthetic")
+    write_images(ds, test_output_path / "synthetic", "synthetic")
     reader = DirectoryImageReader(
-        test_data_path / "synthetic",
+        test_output_path / "synthetic",
         ["X"],
         fmt="synthetic_%Y%m%dT%H%M.nc",
         level={"level": 0},
         **kwargs
     )
     validate_reader(reader, ds[["X"]].isel(level=0))
-    shutil.rmtree(test_data_path / "synthetic")
 
 
-def test_directory_image_reader_discard_attrs(synthetic_test_args):
+def test_directory_image_reader_discard_attrs(synthetic_test_args, test_output_path):
     # tests whether discarding metadata works
     ds, kwargs = synthetic_test_args
-    write_images(ds, test_data_path / "synthetic", "synthetic")
+    write_images(ds, test_output_path / "synthetic", "synthetic")
     reader = DirectoryImageReader(
-        test_data_path / "synthetic",
+        test_output_path / "synthetic",
         ["X", "Y"],
         fmt="synthetic_%Y%m%dT%H%M.nc",
         discard_attrs=True,
@@ -270,18 +238,17 @@ def test_directory_image_reader_discard_attrs(synthetic_test_args):
     ds["X"].attrs = {}
     ds["Y"].attrs = {}
     validate_reader(reader, ds)
-    shutil.rmtree(test_data_path / "synthetic")
 
 
-def test_directory_image_reader_fill_value(synthetic_test_args):
+def test_directory_image_reader_fill_value(synthetic_test_args, test_output_path):
     # tests whether replacing fill values with NaN works
     ds, kwargs = synthetic_test_args
     ds["X"].values[ds.X.values < -0.5] = -9999
     ds["Y"].values[ds.Y.values < -0.5] = -9999
 
-    write_images(ds, test_data_path / "synthetic", "synthetic")
+    write_images(ds, test_output_path / "synthetic", "synthetic")
     reader = DirectoryImageReader(
-        test_data_path / "synthetic",
+        test_output_path / "synthetic",
         ["X", "Y"],
         fmt="synthetic_%Y%m%dT%H%M.nc",
         fill_value=-9999,
@@ -290,15 +257,14 @@ def test_directory_image_reader_fill_value(synthetic_test_args):
     ds["X"].values[ds.X.values == -9999] = np.nan
     ds["Y"].values[ds.Y.values == -9999] = np.nan
     validate_reader(reader, ds)
-    shutil.rmtree(test_data_path / "synthetic")
 
 
-def test_directory_image_reader_nondefault_names(synthetic_test_args):
+def test_directory_image_reader_nondefault_names(synthetic_test_args, test_output_path):
     ds, kwargs = synthetic_test_args
     ds = ds.rename({"lat": "newlat", "lon": "newlon", "time": "newtime"})
-    write_images(ds, test_data_path / "synthetic", "synthetic", dim="newtime")
+    write_images(ds, test_output_path / "synthetic", "synthetic", dim="newtime")
     reader = DirectoryImageReader(
-        test_data_path / "synthetic",
+        test_output_path / "synthetic",
         ["X", "Y"],
         fmt="synthetic_%Y%m%dT%H%M.nc",
         latname="newlat",
@@ -306,57 +272,63 @@ def test_directory_image_reader_nondefault_names(synthetic_test_args):
         timename="newtime",
         **kwargs
     )
-    validate_reader(reader, ds, timename="newtime")
-    shutil.rmtree(test_data_path / "synthetic")
+    validate_reader(reader, ds)
 
 
-def test_directory_image_reader_latlon_from_array(synthetic_test_args):
+def test_directory_image_reader_latlon_from_array(synthetic_test_args, test_output_path):
     ds, kwargs = synthetic_test_args
-    write_images(ds, test_data_path / "synthetic", "synthetic")
+    write_images(ds, test_output_path / "synthetic", "synthetic")
+    dims = ds.X.dims
+    if len(dims) == 3:
+        # regular and curvilinear
+        _, ydim, xdim = dims
+    else:
+        ydim = dims[1]
+        xdim = dims[1]
     reader = DirectoryImageReader(
-        test_data_path / "synthetic",
+        test_output_path / "synthetic",
         ["X", "Y"],
         fmt="synthetic_%Y%m%dT%H%M.nc",
         lat=ds.lat.values,
         lon=ds.lon.values,
+        ydim=ydim,
+        xdim=xdim,
         **kwargs
     )
     validate_reader(reader, ds)
-    shutil.rmtree(test_data_path / "synthetic")
 
 
-def test_directory_image_reader_latlon_from_2d(regular_test_dataset):
+def test_directory_image_reader_latlon_from_2d(regular_test_dataset, test_output_path):
     ds = regular_test_dataset
     orig_ds = ds.copy()
     LON, LAT = np.meshgrid(ds.lon, ds.lat)
     ds["LAT"] = (["lat", "lon"], LAT)
     ds["LON"] = (["lat", "lon"], LON)
     ds = ds.drop_vars(["lat", "lon"])
-    write_images(ds, test_data_path / "synthetic", "synthetic")
+    write_images(ds, test_output_path / "synthetic", "synthetic")
     reader = DirectoryImageReader(
-        test_data_path / "synthetic",
+        test_output_path / "synthetic",
         ["X", "Y"],
         fmt="synthetic_%Y%m%dT%H%M.nc",
         latname="LAT",
-        latdim="lat",
+        ydim="lat",
         lonname="LON",
-        londim="lon",
+        xdim="lon",
     )
     validate_reader(reader, orig_ds)
-    shutil.rmtree(test_data_path / "synthetic")
 
 
-def test_directory_image_reader_landmask(synthetic_test_args):
+def test_directory_image_reader_landmask(synthetic_test_args, test_output_path):
     # tests whether replacing fill values with NaN works
     ds, kwargs = synthetic_test_args
     ds["landmask"] = ds.X.isel(time=0) > -0.5
     ds["X"] = ds.X.where(ds.landmask)
     ds["Y"] = ds.X.where(ds.landmask)
-    write_images(ds, test_data_path / "synthetic", "synthetic")
+    write_images(ds, test_output_path / "synthetic", "synthetic")
 
     # test with array as landmask
     reader = DirectoryImageReader(
-        test_data_path / "synthetic",
+        test_output_path / "synthetic",
         ["X", "Y"],
         fmt="synthetic_%Y%m%dT%H%M.nc",
         landmask=ds["landmask"],
@@ -366,7 +338,7 @@ def test_directory_image_reader_landmask(synthetic_test_args):
 
     # test with string as landmask
     reader = DirectoryImageReader(
-        test_data_path / "synthetic",
+        test_output_path / "synthetic",
         ["X", "Y"],
         fmt="synthetic_%Y%m%dT%H%M.nc",
         landmask="landmask",
@@ -374,32 +346,30 @@ def test_directory_image_reader_landmask(synthetic_test_args):
     )
     validate_reader(reader, ds[["X", "Y"]])
 
-    shutil.rmtree(test_data_path / "synthetic")
 
 
-def test_directory_image_reader_no_grid(synthetic_test_args):
+def test_directory_image_reader_no_grid(synthetic_test_args, test_output_path):
     ds, kwargs = synthetic_test_args
-    write_images(ds, test_data_path / "synthetic", "synthetic")
+    write_images(ds, test_output_path / "synthetic", "synthetic")
     reader = DirectoryImageReader(
-        test_data_path / "synthetic",
+        test_output_path / "synthetic",
         ["X", "Y"],
         fmt="synthetic_%Y%m%dT%H%M.nc",
         construct_grid=False,
         **kwargs
     )
     validate_reader(reader, ds, grid=False)
-    shutil.rmtree(test_data_path / "synthetic")
 
 
-def test_directory_image_reader_averaging(synthetic_test_args):
+def test_directory_image_reader_averaging(synthetic_test_args, test_output_path):
     ds, kwargs = synthetic_test_args
     newtime = pd.date_range(
         ds.indexes["time"][0], periods=len(ds.indexes["time"]), freq="12H"
     )
     ds = ds.assign_coords({"time": newtime})
-    write_images(ds, test_data_path / "synthetic", "synthetic")
+    write_images(ds, test_output_path / "synthetic", "synthetic")
     reader = DirectoryImageReader(
-        test_data_path / "synthetic",
+        test_output_path / "synthetic",
         ["X", "Y"],
         fmt="synthetic_%Y%m%dT%H%M.nc",
         average="daily",
@@ -408,10 +378,9 @@ def test_directory_image_reader_averaging(synthetic_test_args):
     newds = ds.resample(time="D").mean(keep_attrs=True)
     assert len(newds.time) == (len(ds.time) // 2)
     validate_reader(reader, newds)
-    shutil.rmtree(test_data_path / "synthetic")
 
 
-def test_directory_image_reader_multiple_timesteps(synthetic_test_args):
+def test_directory_image_reader_multiple_timesteps(synthetic_test_args, test_output_path):
     # in this test we write out two timesteps per file and the attempt to read
     # them again, using the "timestamps" keyword argument
     ds, kwargs = synthetic_test_args
@@ -421,24 +390,23 @@ def test_directory_image_reader_multiple_timesteps(synthetic_test_args):
     newtime += pd.Timedelta("6H")
     ds = ds.assign_coords({"time": newtime})
     write_images(
-        ds, test_data_path / "synthetic", "synthetic", stepsize=2, fmt="%Y%m%d"
+        ds, test_output_path / "synthetic", "synthetic", stepsize=2, fmt="%Y%m%d"
     )
     reader = DirectoryImageReader(
-        test_data_path / "synthetic",
+        test_output_path / "synthetic",
         ["X", "Y"],
         fmt="synthetic_%Y%m%d.nc",
         timestamps=[pd.Timedelta("6H"), pd.Timedelta("18H")],
         **kwargs
     )
     ntime = len(reader.timestamps)
-    assert len(reader._file_tstamp_map) == ntime // 2
-    for tstamps in reader._file_tstamp_map.values():
+    assert len(reader.blockreader._file_tstamp_map) == ntime // 2
+    for tstamps in reader.blockreader._file_tstamp_map.values():
         assert len(tstamps) == 2
     validate_reader(reader, ds)
-    shutil.rmtree(test_data_path / "synthetic")
 
 
-def test_directory_image_reader_multiple_timesteps_transposed(synthetic_test_args):
+def test_directory_image_reader_multiple_timesteps_transposed(synthetic_test_args, test_output_path):
     # same as multiple_timests test, but now with time as last dimension in the images
     ds, kwargs = synthetic_test_args
     newtime = pd.date_range(
@@ -448,10 +416,10 @@ def test_directory_image_reader_multiple_timesteps_transposed(synthetic_test_arg
     ds = ds.assign_coords({"time": newtime})
     ds = ds.transpose(..., "time")
     write_images(
-        ds, test_data_path / "synthetic", "synthetic", stepsize=2, fmt="%Y%m%d"
+        ds, test_output_path / "synthetic", "synthetic", stepsize=2, fmt="%Y%m%d"
     )
     reader = DirectoryImageReader(
-        test_data_path / "synthetic",
+        test_output_path / "synthetic",
         ["X", "Y"],
         fmt="synthetic_%Y%m%d.nc",
         transpose=("time", ...),  # we have to revert the transpose operation
@@ -460,7 +428,6 @@ def test_directory_image_reader_multiple_timesteps_transposed(synthetic_test_arg
     )
     ds = ds.transpose("time", ...)
     validate_reader(reader, ds)
-    shutil.rmtree(test_data_path / "synthetic")
 
 
 def _write_multistep_files(ds, directory, drop_time=False):
@@ -474,7 +441,7 @@ def _write_multistep_files(ds, directory, drop_time=False):
     ds2.to_netcdf(directory / time[4].strftime("synthetic_%Y%m%d.nc"))
 
 
-def test_directory_image_reader_multiple_timesteps_subset(synthetic_test_args):
+def test_directory_image_reader_multiple_timesteps_subset(synthetic_test_args, test_output_path):
     # here we test if it works to only read a subset of timesteps from files
     # with multiple timesteps
     ds, kwargs = synthetic_test_args
@@ -482,9 +449,9 @@ def test_directory_image_reader_multiple_timesteps_subset(synthetic_test_args):
         ds.indexes["time"][0], periods=len(ds.indexes["time"]), freq="6H"
     )
     ds = ds.assign_coords(time=newtime)
-    _write_multistep_files(ds, test_data_path / "synthetic")
+    _write_multistep_files(ds, test_output_path / "synthetic")
     reader = DirectoryImageReader(
-        test_data_path / "synthetic",
+        test_output_path / "synthetic",
         ["X", "Y"],
         fmt="synthetic_%Y%m%d.nc",
         timestamps=[
@@ -497,10 +464,9 @@ def test_directory_image_reader_multiple_timesteps_subset(synthetic_test_args):
     )
     ds = ds.isel(time=slice(0, 8))
     validate_reader(reader, ds)
-    shutil.rmtree(test_data_path / "synthetic")
 
 
-def test_directory_image_reader_multiple_timesteps_subset_notime(synthetic_test_args):
+def test_directory_image_reader_multiple_timesteps_subset_notime(synthetic_test_args, test_output_path):
     # here we test if it works to only read a subset of timesteps from files
     # with multiple timesteps
     ds, kwargs = synthetic_test_args
@@ -508,9 +474,9 @@ def test_directory_image_reader_multiple_timesteps_subset_notime(synthetic_test_
         ds.indexes["time"][0], periods=len(ds.indexes["time"]), freq="6H"
     )
     ds = ds.assign_coords(time=newtime)
-    _write_multistep_files(ds, test_data_path / "synthetic", drop_time=True)
+    _write_multistep_files(ds, test_output_path / "synthetic", drop_time=True)
     reader = DirectoryImageReader(
-        test_data_path / "synthetic",
+        test_output_path / "synthetic",
         ["X", "Y"],
         fmt="synthetic_%Y%m%d.nc",
         timestamps=[
@@ -523,4 +489,3 @@ def test_directory_image_reader_multiple_timesteps_subset_notime(synthetic_test_
     )
     ds = ds.isel(time=slice(0, 8))
     validate_reader(reader, ds)
-    shutil.rmtree(test_data_path / "synthetic")
